本发明涉及Web平台开发领域，尤其涉及一种实现Fetch请求失败自动重试的方法。


随着HTML5技术的快速发展，以往的前端与服务器交互方法XMLHttpRequest慢慢被Fetch代替。
Fetch天生支持Promise,并能够很容易被HTML5其他技术所使用，比如ServiceWorker。
然而在大中型网站中，经常会碰到由于网络波动，服务器不稳定等问题，导致前端发起的请求会失败，
这时用户只能手动再发起一次请求，极大的影响了用户体验。因此，有必要提出一种实现Fetch请求失败自动重试的方法，
以实现Fetch请求失败自动重试。

本发明的目的在于提供一种实现Fetch请求自动重试的方法。
为达到上述技术目的，本发明提供了一种实现Fetch请求失败自动重试的方法，包括以下步骤:

S101、获取待发送的Fetch请求原生参数，设置重试参数并与Fetch请求原生参数合并。
S102、初始化Promise任务队列，将Fetch请求，不处理函数以及重试拦截器依次加入到Promise任务队列中。
S103、开始执行Promise任务队列，等待Fetch请求是否成功。
若是，执行S104，否则执行S105。
S104、将服务器返回的结果传递给不处理函数，执行不处理函数，然后结束请求。
S105、将服务器返回的错误信息交给重试拦截器处理函数，判断重试次数是否大于1以及是否满足重试条件
若是,执行S106,否则结束请求。
S106、根据延时时间设置延时函数，等待延时函数执行完成。
完成后重新执行S103
//如果Fetch请求失败，会根据重试条件以及请求重试次数决定重试拦截器是否要重新发送请求。
优选地，所述重试参数形式如下:
  {
     "retryCount": 3 //Fetch请求重试次数
     "delay"： 3000 //Fetch请求每隔多少时间重试一次，单位为秒
     "condition": function (error) { // Fetch请求重试条件判断函数，condition函数默认为返回true的函数。
        //error:请求失败返回的错误数据
        //TODO:根据传入的error判断是否要重试，return true表示重试，否则不重试
     }
  }
优选地，所述Promise任务队列内容如下:
  [
   Fetch,       //Fetch请求
   onFulfilled, //Fetch请求成功不处理函数。
   onRejected   //重试拦截器处理函数，Fetch请求失败会被该函数拦截并处理。
  ]
优选地，所述步骤S102具体为:
创建一个空数组为promiseTask，promiseTask是一个队列，满足队列先入先出的特性;
获取Fetch请求，将其加入到promiseTask任务队列中;
设置不处理函数，该函数只有在Fetch请求成功的情况下才会被执行，该函数直接会返回响应结果，不会做任何其他处理；
将不处理函数加入到promiseTask任务队列中;
设置重试拦截器函数，该函数会在Fetch请求失败后被执行，重试拦截器函数决定了Fetch请求是否能够被重新发送;
将重试拦截器函数加入到Promise任务队列中;

优选地，所述步骤S103具体为:
将Promise任务队列中的任务依次出队列，并通过Promise.then(onFulfilled,onRejected)将任务依次连接起来;
Fetch请求将会被第一个执行;
当Fetch请求成功时，会执行不处理函数，也就是onFulfilled，该函数直接返回响应结果，请求完成;
当Fetch请求发生错误时，会执行重试拦截器处理函数，也就是onRejected;
优选地，所述步骤S105具体为:
获取服务器返回的错误信息为error;
获取重试次数为retrycount，重试条件判断函数为condition;
如果retrycount小于1，则表示重试次数用完，将会直接结束请求;
如果condition执行结果没有返回true，也会直接结束请求;
如果重试次数大于1而且condition执行结果返回true，则将retrycount数减一
优选地，所属步骤S106具体为:
获取延时时间为delay;
设置延迟函数，将延迟函数的timeout值设置为delay；
执行延迟函数并等待延迟函数的完成；


本发明的技术效果和优点：本发明能够实现Fetch请求在失败的情况下自动重试的方法，不需要用户手动去重试请求，提升了用户体验，
具有实现方式简单，且配置参数灵活，支持配置重试次数，单次重试延迟时间，自定义重试条件多种配置参数的优点。

图1为本申请实施例提供的实现Fetch请求失败重试的方法的流程示意图;
图2为本申请实施例提供的重试拦截器执行过程的流程示意图;


为了使本技术领域的人员更好地理解本申请中的技术方案，下面将结合本申请实施例中的附图，
对本申请实施例中的技术方案进行清楚、完整地描述，
显然，所描述的实施 例仅仅是本申请一部分实施例，
而不是全部的实施例。基于本申请中的实施例，本领域普通技术人员在没有作出创造性劳动前提下所获得的所有其他实施例，都应当属于本申请保护的范围。

下面结合附图对本发明实施例所提供的一种实现Fetch请求失败自动重试的方法进行详细说明。
如图1所示，本发明实施例公开了一种实现Fetch请求失败自动重试的方法，包括以下步
骤：
S101、获取待发送的Fetch请求原生参数，设置重试参数并与Fetch请求原生参数合并。

S102、初始化Promise任务队列，将Fetch请求，不处理函数以及重试拦截器依次加入到Promise任务队列中。
创建一个空数组为promiseTask，promiseTask是一个队列，满足队列先入先出的特性;
获取Fetch请求，将其加入到promiseTask任务队列中;
设置不处理函数，该函数只有在Fetch请求成功的情况下才会被执行，该函数直接会返回响应结果，不会做任何其他处理；
将不处理函数加入到promiseTask任务队列中;
设置重试拦截器函数，该函数会在Fetch请求失败后被执行，重试拦截器函数决定了Fetch请求是否能够被重新发送;
将重试拦截器函数加入到Promise任务队列中;

S103、开始执行Promise任务队列，等待Fetch请求是否成功。
若是，执行S104，否则执行S105。
将Promise任务队列中的任务依次出队列，并通过Promise.then(onFulfilled,onRejected)将任务依次连接起来;
Fetch请求将会被第一个执行;
当Fetch请求成功时，会执行不处理函数，也就是onFulfilled，该函数直接返回响应结果，请求完成;
当Fetch请求发生错误时，会执行重试拦截器处理函数，也就是onRejected;
优选地，所述步骤S105具体为:
获取服务器返回的错误信息为error;
获取重试次数为retrycount，重试条件判断函数为condition;
如果retrycount小于1，则表示重试次数用完，将会直接结束请求;
如果condition执行结果没有返回true，也会直接结束请求;
如果重试次数大于1而且condition执行结果返回true，则将retrycount数减一
优选地，所属步骤S106具体为:
获取延时时间为delay;
设置延迟函数，将延迟函数的timeout值设置为delay；
执行延迟函数并等待延迟函数的完成；

S104、将服务器返回的结果传递给不处理函数，执行不处理函数，然后结束请求。

S105、将服务器返回的错误信息交给重试拦截器处理函数，判断重试次数是否大于1以及是否满足重试条件
若是,执行S106,否则结束请求。
S106、根据延时时间设置延时函数，等待延时函数执行完成。
完成后重新执行S103

以上所述，仅为本发明较佳的具体实施方式，但本发明的保护范围并不局限于此，
任何熟悉本技术领域的技术人员在本发明揭露的技术范围内，根据本发明的技术方案及其
发明构思加以等同替换或改变，都应涵盖在本发明的保护范围之内。